==> create-fly-1.21.10-6.0.8-4-sources.jar: com/zurrtum/create/client/foundation/blockEntity/behaviour/filtering/FilteringRenderer.java <==
package com.zurrtum.create.client.foundation.blockEntity.behaviour.filtering;

import com.zurrtum.create.AllBlocks;
import com.zurrtum.create.catnip.data.Iterate;
import com.zurrtum.create.catnip.data.Pair;
import com.zurrtum.create.client.AllSpecialTextures;
import com.zurrtum.create.client.Create;
import com.zurrtum.create.client.catnip.outliner.Outliner;
import com.zurrtum.create.client.content.logistics.factoryBoard.FactoryPanelBehaviour;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBox;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBox.ItemValueBox;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBoxRenderer;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBoxTransform;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBoxTransform.Sided;
import com.zurrtum.create.content.logistics.factoryBoard.FactoryPanelBlockEntity;
import com.zurrtum.create.content.logistics.filter.FilterItem;
import com.zurrtum.create.foundation.blockEntity.SmartBlockEntity;
import com.zurrtum.create.foundation.blockEntity.behaviour.BlockEntityBehaviour;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.class_10442;
import net.minecraft.class_10444;
import net.minecraft.class_11659;
import net.minecraft.class_1268;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_238;
import net.minecraft.class_239;
import net.minecraft.class_243;
import net.minecraft.class_2561;
import net.minecraft.class_2680;
import net.minecraft.class_310;
import net.minecraft.class_3965;
import net.minecraft.class_4587;
import net.minecraft.class_5250;
import net.minecraft.class_638;
import net.minecraft.class_811;

public class FilteringRenderer {
    public static void tick(class_310 mc) {
        class_239 target = mc.field_1765;
        if (!(target instanceof class_3965 result))
            return;

        class_638 world = mc.field_1687;
        class_2338 pos = result.method_17777();
        class_2680 state = world.method_8320(pos);

        if (mc.field_1724.method_5715())
            return;
        if (!(world.method_8321(pos) instanceof SmartBlockEntity sbe))
            return;

        class_1799 mainhandItem = mc.field_1724.method_5998(class_1268.field_5808);

        List<FilteringBehaviour<?>> behaviours;
        if (sbe instanceof FactoryPanelBlockEntity fpbe) {
            behaviours = FactoryPanelBehaviour.allBehaviours(fpbe);
        } else {
            FilteringBehaviour<?> behaviour = sbe.getBehaviour(FilteringBehaviour.TYPE);
            if (behaviour instanceof SidedFilteringBehaviour sidedBehaviour) {
                behaviour = sidedBehaviour.get(result.method_17780());
            }
            if (behaviour == null) {
                return;
            }
            behaviours = List.of(behaviour);
        }

        for (FilteringBehaviour<?> behaviour : behaviours) {
            if (!behaviour.isActive())
                continue;
            if (behaviour.slotPositioning instanceof ValueBoxTransform.Sided)
                ((Sided) behaviour.slotPositioning).fromSide(result.method_17780());
            if (!behaviour.slotPositioning.shouldRender(state))
                continue;
            if (!behaviour.mayInteract(mc.field_1724))
                continue;

            class_1799 filter = behaviour.getFilter();
            boolean isFilterSlotted = filter.method_7909() instanceof FilterItem;
            boolean showCount = behaviour.isCountVisible();
            class_2561 label = behaviour.getLabel();
            boolean hit = behaviour.slotPositioning.testHit(world, pos, state, target.method_17784().method_1020(class_243.method_24954(pos)));

            class_238 emptyBB = new class_238(class_243.field_1353, class_243.field_1353);
            class_238 bb = isFilterSlotted ? emptyBB.method_1009(.45f, .31f, .2f) : emptyBB.method_1014(.25f);

            ValueBox box = new ItemValueBox(label, bb, pos, filter, behaviour.getCountLabelForValueBox());
            box.passive(!hit || behaviour.bypassesInput(mainhandItem));

            Outliner.getInstance().showOutline(Pair.of("filter" + behaviour.netId(), pos), box.transform(behaviour.slotPositioning))
                .lineWidth(1 / 64f).withFaceTexture(hit ? AllSpecialTextures.THIN_CHECKERED : null).highlightFace(result.method_17780());

            if (!hit)
                continue;

            List<class_5250> tip = new ArrayList<>();
            tip.add(label.method_27661());
            tip.add(behaviour.getTip());
            if (showCount)
                tip.add(behaviour.getAmountTip());

            Create.VALUE_SETTINGS_HANDLER.showHoverTip(mc, tip);
        }
    }

    @Nullable
    public static FilterRenderState getFilterRenderState(
        SmartBlockEntity be,
        class_2680 blockState,
        class_10442 itemModelManager,
        double distance
    ) {
        if (be instanceof FactoryPanelBlockEntity) {
            List<SingleFilterRenderState> list = null;
            int count = 0;
            boolean check = distance != -1;
            for (BlockEntityBehaviour<?> behaviour : be.getAllBehaviours()) {
                if (behaviour instanceof FactoryPanelBehaviour factoryPanelBehaviour) {
                    if (factoryPanelBehaviour.behaviour != null && factoryPanelBehaviour.isActive()) {
                        if (check) {
                            if (isOutOfRange(factoryPanelBehaviour, distance)) {
                                return null;
                            }
                            check = false;
                        }
                        class_1799 filter = factoryPanelBehaviour.getFilter();
                        ValueBoxTransform slotPositioning = factoryPanelBehaviour.getSlotPositioning();
                        if (!filter.method_7960() && slotPositioning.shouldRender(blockState)) {
                            if (list == null) {
                                list = new ArrayList<>(4);
                            }
                            list.add(SingleFilterRenderState.create(slotPositioning, itemModelManager, filter, be.method_10997()));
                        }
                    }
                    if (++count == 4) {
                        break;
                    }
                }
            }
            return list == null ? null : new FactoryPanelRenderState(list);
        }
        FilteringBehaviour<?> behaviour = be.getBehaviour(FilteringBehaviour.TYPE);
        if (behaviour == null) {
            return null;
        }
        if (!be.isVirtual() && (behaviour.behaviour == null || !behaviour.isActive() || isOutOfRange(behaviour, distance))) {
            return null;
        }
        if (behaviour instanceof SidedFilteringBehaviour sidedFilteringBehaviour) {
            return SidedFilterRenderState.create(sidedFilteringBehaviour, blockState, itemModelManager, be.method_10997());
        }
        class_1799 filter = behaviour.getFilter();
        if (filter.method_7960()) {
            return null;
        }
        ValueBoxTransform slotPositioning = behaviour.getSlotPositioning();
        if (slotPositioning instanceof Sided sided) {
            return SidedSingleFilterRenderState.create(sided, blockState, itemModelManager, filter, be.method_10997());
        }
        return SingleFilterRenderState.create(slotPositioning, itemModelManager, filter, be.method_10997());
    }

    private static boolean isOutOfRange(FilteringBehaviour<?> behaviour, double distance) {
        if (distance == -1) {
            return false;
        }
        float max = behaviour.getRenderDistance();
        return max * max < distance;
    }

    public interface FilterRenderState {
        void render(class_2680 blockState, class_11659 queue, class_4587 ms, int light);
    }

    public record FactoryPanelRenderState(List<SingleFilterRenderState> states) implements FilterRenderState {
        @Override
        public void render(class_2680 blockState, class_11659 queue, class_4587 ms, int light) {
            for (SingleFilterRenderState state : states) {
                state.render(blockState, queue, ms, light);
            }
        }
    }

    public record SingleFilterRenderState(ValueBoxTransform slotPositioning, class_10444 state, float offset) implements FilterRenderState {
        public static SingleFilterRenderState create(
            ValueBoxTransform slotPositioning,
            class_10442 itemModelManager,
            class_1799 stack,
            class_1937 world
        ) {
            class_10444 renderState = new class_10444();
            renderState.field_55337 = class_811.field_4319;
            itemModelManager.method_65596(renderState, stack, class_811.field_4319, world, null, 0);
            return new SingleFilterRenderState(slotPositioning, renderState, ValueBoxRenderer.customZOffset(stack.method_7909()));
        }

        @Override
        public void render(class_2680 blockState, class_11659 queue, class_4587 ms, int light) {
            ms.method_22903();
            slotPositioning.transform(blockState, ms);
            ValueBoxRenderer.renderItemIntoValueBox(state, queue, ms, light, offset);
            ms.method_22909();
        }
    }

    public record SidedSingleFilterRenderState(
        Sided sided, class_2350 side, class_10444 state, Float offset, List<class_2350> sides
    ) implements FilterRenderState {
        public static SidedSingleFilterRenderState create(
            Sided sided,
            class_2680 blockState,
            class_10442 itemModelManager,
            class_1799 filter,
            class_1937 world
        ) {
            class_10444 renderState = new class_10444();
            Float offset;
            if (blockState.method_27852(AllBlocks.CONTRAPTION_CONTROLS)) {
                renderState.field_55337 = class_811.field_4317;
                offset = null;
            } else {
                renderState.field_55337 = class_811.field_4319;
                offset = ValueBoxRenderer.customZOffset(filter.method_7909());
            }
            itemModelManager.method_65596(renderState, filter, renderState.field_55337, world, null, 0);
            class_2350 side = sided.getSide();
            List<class_2350> sides = new ArrayList<>();
            for (class_2350 direction : Iterate.directions) {
                sided.fromSide(direction);
                if (sided.shouldRender(blockState)) {
                    sides.add(direction);
                }
            }
            sided.fromSide(side);
            return new SidedSingleFilterRenderState(sided, side, renderState, offset, sides);
        }

        @Override
        public void render(class_2680 blockState, class_11659 queue, class_4587 ms, int light) {
            boolean flat = offset == null;
            for (class_2350 side : sides) {
                ms.method_22903();
                sided.fromSide(side);
                sided.transform(blockState, ms);
                if (flat) {
                    ValueBoxRenderer.renderFlatItemIntoValueBox(state, queue, ms, light);
                } else {
                    ValueBoxRenderer.renderItemIntoValueBox(state, queue, ms, light, offset);
                }
                ms.method_22909();
            }
            sided.fromSide(side);
        }
    }

    public record SidedFilterRenderState(
        Sided slotPositioning, class_2350 side, List<BoxRenderState> boxes
    ) implements FilterRenderState {
        @Nullable
        public static FilterRenderState create(
            SidedFilteringBehaviour behaviour,
            class_2680 blockState,
            class_10442 itemModelManager,
            class_1937 world
        ) {
            boolean flat = blockState.method_27852(AllBlocks.CONTRAPTION_CONTROLS);
            Sided sided = behaviour.getSlotPositioning();
            List<SidedFilterRenderState.BoxRenderState> boxes = new ArrayList<>();
            class_2350 side = sided.getSide();
            for (class_2350 direction : Iterate.directions) {
                class_1799 filter = behaviour.getFilter(direction);
                if (filter.method_7960())
                    continue;
                sided.fromSide(direction);
                if (!sided.shouldRender(blockState))
                    continue;
                class_10444 renderState = new class_10444();
                if (flat) {
                    renderState.field_55337 = class_811.field_4317;
                    itemModelManager.method_65596(renderState, filter, renderState.field_55337, world, null, 0);
                    boxes.add(new FlatBoxState(direction, renderState));
                } else {
                    renderState.field_55337 = class_811.field_4319;
                    itemModelManager.method_65596(renderState, filter, renderState.field_55337, world, null, 0);
                    boxes.add(new BoxState(direction, renderState, ValueBoxRenderer.customZOffset(filter.method_7909())));
                }
            }
            sided.fromSide(side);
            return boxes.isEmpty() ? null : new SidedFilterRenderState(sided, side, boxes);
        }

        @Override
        public void render(class_2680 blockState, class_11659 queue, class_4587 ms, int light) {
            for (BoxRenderState box : boxes) {
                ms.method_22903();
                slotPositioning.fromSide(box.side());
                slotPositioning.transform(blockState, ms);
                box.render(queue, ms, light);
                ms.method_22909();
            }
            slotPositioning.fromSide(side);
        }

        public interface BoxRenderState {
            class_2350 side();

            void render(class_11659 queue, class_4587 ms, int light);
        }

        record FlatBoxState(class_2350 side, class_10444 state) implements BoxRenderState {
            @Override
            public void render(class_11659 queue, class_4587 ms, int light) {
                ValueBoxRenderer.renderFlatItemIntoValueBox(state, queue, ms, light);
            }
        }

        record BoxState(class_2350 side, class_10444 state, float offset) implements BoxRenderState {
            @Override
            public void render(class_11659 queue, class_4587 ms, int light) {
                ValueBoxRenderer.renderItemIntoValueBox(state, queue, ms, light, offset);
            }
        }
    }
}

==> create-fly-6.0.7-1.21.8-19-sources.jar: com/zurrtum/create/client/foundation/blockEntity/behaviour/filtering/FilteringRenderer.java <==
package com.zurrtum.create.client.foundation.blockEntity.behaviour.filtering;

import com.zurrtum.create.AllBlocks;
import com.zurrtum.create.catnip.data.Iterate;
import com.zurrtum.create.catnip.data.Pair;
import com.zurrtum.create.catnip.math.VecHelper;
import com.zurrtum.create.client.AllSpecialTextures;
import com.zurrtum.create.client.Create;
import com.zurrtum.create.client.catnip.outliner.Outliner;
import com.zurrtum.create.client.content.logistics.factoryBoard.FactoryPanelBehaviour;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBox;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBox.ItemValueBox;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBoxRenderer;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBoxTransform;
import com.zurrtum.create.client.foundation.blockEntity.behaviour.ValueBoxTransform.Sided;
import com.zurrtum.create.content.logistics.factoryBoard.FactoryPanelBlockEntity;
import com.zurrtum.create.content.logistics.filter.FilterItem;
import com.zurrtum.create.foundation.blockEntity.SmartBlockEntity;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.class_1268;
import net.minecraft.class_1297;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_238;
import net.minecraft.class_239;
import net.minecraft.class_243;
import net.minecraft.class_2561;
import net.minecraft.class_2680;
import net.minecraft.class_310;
import net.minecraft.class_3965;
import net.minecraft.class_4587;
import net.minecraft.class_4597;
import net.minecraft.class_5250;
import net.minecraft.class_638;

public class FilteringRenderer {

    public static void tick(class_310 mc) {
        class_239 target = mc.field_1765;
        if (!(target instanceof class_3965 result))
            return;

        class_638 world = mc.field_1687;
        class_2338 pos = result.method_17777();
        class_2680 state = world.method_8320(pos);

        if (mc.field_1724.method_5715())
            return;
        if (!(world.method_8321(pos) instanceof SmartBlockEntity sbe))
            return;

        class_1799 mainhandItem = mc.field_1724.method_5998(class_1268.field_5808);

        List<FilteringBehaviour<?>> behaviours;
        if (sbe instanceof FactoryPanelBlockEntity fpbe) {
            behaviours = FactoryPanelBehaviour.allBehaviours(fpbe);
        } else {
            FilteringBehaviour<?> behaviour = sbe.getBehaviour(FilteringBehaviour.TYPE);
            if (behaviour instanceof SidedFilteringBehaviour sidedBehaviour) {
                behaviour = sidedBehaviour.get(result.method_17780());
            }
            if (behaviour == null) {
                return;
            }
            behaviours = List.of(behaviour);
        }

        for (FilteringBehaviour<?> behaviour : behaviours) {
            if (!behaviour.isActive())
                continue;
            if (behaviour.slotPositioning instanceof ValueBoxTransform.Sided)
                ((Sided) behaviour.slotPositioning).fromSide(result.method_17780());
            if (!behaviour.slotPositioning.shouldRender(world, pos, state))
                continue;
            if (!behaviour.mayInteract(mc.field_1724))
                continue;

            class_1799 filter = behaviour.getFilter();
            boolean isFilterSlotted = filter.method_7909() instanceof FilterItem;
            boolean showCount = behaviour.isCountVisible();
            class_2561 label = behaviour.getLabel();
            boolean hit = behaviour.slotPositioning.testHit(world, pos, state, target.method_17784().method_1020(class_243.method_24954(pos)));

            class_238 emptyBB = new class_238(class_243.field_1353, class_243.field_1353);
            class_238 bb = isFilterSlotted ? emptyBB.method_1009(.45f, .31f, .2f) : emptyBB.method_1014(.25f);

            ValueBox box = new ItemValueBox(label, bb, pos, filter, behaviour.getCountLabelForValueBox());
            box.passive(!hit || behaviour.bypassesInput(mainhandItem));

            Outliner.getInstance().showOutline(Pair.of("filter" + behaviour.netId(), pos), box.transform(behaviour.slotPositioning))
                .lineWidth(1 / 64f).withFaceTexture(hit ? AllSpecialTextures.THIN_CHECKERED : null).highlightFace(result.method_17780());

            if (!hit)
                continue;

            List<class_5250> tip = new ArrayList<>();
            tip.add(label.method_27661());
            tip.add(behaviour.getTip());
            if (showCount)
                tip.add(behaviour.getAmountTip());

            Create.VALUE_SETTINGS_HANDLER.showHoverTip(mc, tip);
        }
    }

    public static void renderOnBlockEntity(
        SmartBlockEntity be,
        float partialTicks,
        class_4587 ms,
        class_4597 buffer,
        int light,
        int overlay
    ) {

        if (be == null || be.method_11015())
            return;

        class_1937 level = be.method_10997();
        class_2338 blockPos = be.method_11016();

        List<FilteringBehaviour<?>> behaviours;
        if (be instanceof FactoryPanelBlockEntity fpbe) {
            behaviours = FactoryPanelBehaviour.allBehaviours(fpbe);
        } else {
            FilteringBehaviour<?> behaviour = be.getBehaviour(FilteringBehaviour.TYPE);
            if (behaviour == null) {
                return;
            }
            behaviours = List.of(behaviour);
        }

        for (FilteringBehaviour<?> behaviour : behaviours) {
            if (!be.isVirtual()) {
                class_1297 cameraEntity = class_310.method_1551().field_1719;
                if (cameraEntity != null && level == cameraEntity.method_37908()) {
                    float max = behaviour.getRenderDistance();
                    if (cameraEntity.method_19538().method_1025(VecHelper.getCenterOf(blockPos)) > (max * max)) {
                        continue;
                    }
                }
            }

            if (behaviour.behaviour == null || !behaviour.isActive())
                continue;
            if (behaviour.getFilter().method_7960() && !(behaviour instanceof SidedFilteringBehaviour))
                continue;

            ValueBoxTransform slotPositioning = behaviour.slotPositioning;
            class_2680 blockState = be.method_11010();

            if (slotPositioning instanceof Sided sided) {
                class_2350 side = sided.getSide();
                for (class_2350 d : Iterate.directions) {
                    class_1799 filter = behaviour.getFilter(d);
                    if (filter.method_7960())
                        continue;

                    sided.fromSide(d);
                    if (!slotPositioning.shouldRender(level, blockPos, blockState))
                        continue;

                    ms.method_22903();
                    slotPositioning.transform(level, blockPos, blockState, ms);
                    if (blockState.method_27852(AllBlocks.CONTRAPTION_CONTROLS))
                        ValueBoxRenderer.renderFlatItemIntoValueBox(filter, ms, buffer, light, overlay);
                    else
                        ValueBoxRenderer.renderItemIntoValueBox(filter, ms, buffer, light, overlay);
                    ms.method_22909();
                }
                sided.fromSide(side);
            } else if (slotPositioning.shouldRender(level, blockPos, blockState)) {
                ms.method_22903();
                slotPositioning.transform(level, blockPos, blockState, ms);
                ValueBoxRenderer.renderItemIntoValueBox(behaviour.getFilter(), ms, buffer, light, overlay);
                ms.method_22909();
            }
        }
    }

}
